@@ -27,41 +27,64 @@
 
 typedef struct
 {
-  GtkWidget		*file_radio;
+  GtkWidget		*type_label;
+  GtkWidget		*type_combo;
+
+  GtkWidget		*autodetect_vbox;
   GtkWidget		*location_label;
   GtkWidget		*location_entry;
-  GtkWidget		*browse;
-  GtkWidget		*pop3_radio;
-  GtkWidget		*hostname_label;
-  GtkWidget		*hostname_entry;
-  GtkWidget		*port_label;
-  GtkWidget		*port_spin;
-  GtkWidget		*username_label;
-  GtkWidget		*username_entry;
-  GtkWidget		*password_label;
-  GtkWidget		*password_entry;
+  GtkWidget		*browse_button;
+
+  GtkWidget		*pop3_vbox;
+  GtkWidget		*pop3_hostname_label;
+  GtkWidget		*pop3_hostname_entry;
+  GtkWidget		*pop3_port_label;
+  GtkWidget		*pop3_port_spin;
+  GtkWidget		*pop3_username_label;
+  GtkWidget		*pop3_username_entry;
+  GtkWidget		*pop3_password_label;
+  GtkWidget		*pop3_password_entry;
+
+  GtkWidget		*gmail_vbox;
+  GtkWidget		*gmail_username_label;
+  GtkWidget		*gmail_username_entry;
+  GtkWidget		*gmail_password_label;
+  GtkWidget		*gmail_password_entry;
+
+  GtkWidget		*unsupported_label;
 
   MNMailboxPropertiesDialogMode		mode;
   char					*uri;
   GtkWidget				*apply_button;
   GtkWidget				*accept_button;
+  GtkListStore				*store;
 } Private;
 
-typedef enum
-{
-  RADIO_FILE,
-  RADIO_POP3
-} Radio;
-
+typedef enum {
+  TYPE_AUTODETECT,
+  TYPE_POP3,
+  TYPE_GMAIL
+} Type;
+    
+enum {
+  COLUMN_TYPE,
+  COLUMN_TEXT,
+  COLUMN_STYLE,
+  N_COLUMNS
+};
+    
 /*** variables ***************************************************************/
 
-static Radio selected_radio = RADIO_FILE;
 static char *current_folder_uri = NULL;
+static Type selected_type = TYPE_AUTODETECT;
 
 /*** functions ***************************************************************/
 
 static void mn_mailbox_properties_dialog_private_free (Private *private);
 static gboolean mn_mailbox_properties_dialog_is_complete (MNMailboxPropertiesDialog *dialog);
+static Type mn_mailbox_properties_dialog_get_active_type (MNMailboxPropertiesDialog *dialog);
+static void mn_mailbox_properties_dialog_set_active_type (MNMailboxPropertiesDialog *dialog,
+							  Type type);
 static void mn_mailbox_properties_dialog_update_sensitivity (MNMailboxPropertiesDialog *dialog);
 static void mn_mailbox_properties_dialog_set_uri_internal (MNMailboxPropertiesDialog *dialog,
 							   const char *uri);
@@ -78,23 +101,33 @@ mn_mailbox_properties_dialog_new (GtkWindow *parent,
   MNMailboxPropertiesDialog *dialog;
   Private *private;
   GtkSizeGroup *size_group;
+  GtkTreeIter iter;
+  GtkCellRenderer *renderer;
 
   private = g_new0(Private, 1);
   mn_create_interface("mailbox-properties",
 		      "dialog", (GtkWidget **) &dialog,
-		      "file_radio", &private->file_radio,
+		      "type_label", &private->type_label,
+		      "type_combo", &private->type_combo,
+		      "autodetect_vbox", &private->autodetect_vbox,
 		      "location_label", &private->location_label,
 		      "location_entry", &private->location_entry,
-		      "browse", &private->browse,
-		      "pop3_radio", &private->pop3_radio,
-		      "hostname_label", &private->hostname_label,
-		      "hostname_entry", &private->hostname_entry,
-		      "port_label", &private->port_label,
-		      "port_spin", &private->port_spin,
-		      "username_label", &private->username_label,
-		      "username_entry", &private->username_entry,
-		      "password_label", &private->password_label,
-		      "password_entry", &private->password_entry,
+		      "browse_button", &private->browse_button,
+		      "pop3_vbox", &private->pop3_vbox,
+		      "pop3_hostname_label", &private->pop3_hostname_label,
+		      "pop3_hostname_entry", &private->pop3_hostname_entry,
+		      "pop3_port_label", &private->pop3_port_label,
+		      "pop3_port_spin", &private->pop3_port_spin,
+		      "pop3_username_label", &private->pop3_username_label,
+		      "pop3_username_entry", &private->pop3_username_entry,
+		      "pop3_password_label", &private->pop3_password_label,
+		      "pop3_password_entry", &private->pop3_password_entry,
+		      "gmail_vbox", &private->gmail_vbox,
+		      "gmail_username_label", &private->gmail_username_label,
+		      "gmail_username_entry", &private->gmail_username_entry,
+		      "gmail_password_label", &private->gmail_password_label,
+		      "gmail_password_entry", &private->gmail_password_entry,
+		      "unsupported_label", &private->unsupported_label,
 		      NULL);
   private->mode = mode;
 
@@ -104,14 +137,19 @@ mn_mailbox_properties_dialog_new (GtkWindow *parent,
 			 (GDestroyNotify) mn_mailbox_properties_dialog_private_free);
 
   size_group = gtk_size_group_new(GTK_SIZE_GROUP_HORIZONTAL);
+  gtk_size_group_add_widget(size_group, private->type_label);
   gtk_size_group_add_widget(size_group, private->location_label);
-  gtk_size_group_add_widget(size_group, private->hostname_label);
-  gtk_size_group_add_widget(size_group, private->username_label);
+  gtk_size_group_add_widget(size_group, private->pop3_hostname_label);
+  gtk_size_group_add_widget(size_group, private->pop3_username_label);
+  gtk_size_group_add_widget(size_group, private->pop3_password_label);
+  gtk_size_group_add_widget(size_group, private->gmail_username_label);
+  gtk_size_group_add_widget(size_group, private->gmail_password_label);
   g_object_unref(size_group);
 
   if (parent)
     gtk_window_set_transient_for(GTK_WINDOW(dialog), parent);
 
+  gtk_dialog_add_button(GTK_DIALOG(dialog), GTK_STOCK_HELP, GTK_RESPONSE_HELP);
   if (mode == MN_MAILBOX_PROPERTIES_DIALOG_MODE_ADD)
     {
       gtk_window_set_title(GTK_WINDOW(dialog), _("Add a Mailbox"));
@@ -131,29 +169,44 @@ mn_mailbox_properties_dialog_new (GtkWindow *parent,
     g_return_val_if_reached(NULL);
   gtk_widget_grab_default(private->accept_button);
       
-#ifndef WITH_POP3
-  gtk_widget_set_sensitive(private->pop3_radio, FALSE);
-#endif
+  /* finish the combo box */
+
+  private->store = gtk_list_store_new(N_COLUMNS,
+				      G_TYPE_INT,
+				      G_TYPE_STRING,
+				      PANGO_TYPE_STYLE);
+
+  gtk_list_store_append(private->store, &iter);
+  gtk_list_store_set(private->store, &iter,
+		     COLUMN_TYPE, TYPE_AUTODETECT,
+		     COLUMN_TEXT, _("autodetect"),
+		     COLUMN_STYLE, PANGO_STYLE_ITALIC,
+		     -1);
+
+  gtk_list_store_append(private->store, &iter);
+  gtk_list_store_set(private->store, &iter,
+		     COLUMN_TYPE, TYPE_POP3,
+		     COLUMN_TEXT, "POP3",
+		     -1);
+
+  gtk_list_store_append(private->store, &iter);
+  gtk_list_store_set(private->store, &iter,
+		     COLUMN_TYPE, TYPE_GMAIL,
+		     COLUMN_TEXT, "Gmail",
+		     -1);
+
+  renderer = gtk_cell_renderer_text_new();
+  gtk_cell_layout_pack_start(GTK_CELL_LAYOUT(private->type_combo), renderer, TRUE);
+  gtk_cell_layout_set_attributes(GTK_CELL_LAYOUT(private->type_combo), renderer,
+				 "text", COLUMN_TEXT,
+				 "style", COLUMN_STYLE,
+				 NULL);
+
+  gtk_combo_box_set_model(GTK_COMBO_BOX(private->type_combo), GTK_TREE_MODEL(private->store));
 
   if (mode == MN_MAILBOX_PROPERTIES_DIALOG_MODE_ADD)
-    {
-      GtkWidget *default_radio;
-      
-      switch (selected_radio)
-	{
-	case RADIO_FILE:	default_radio = private->file_radio; break;
-	case RADIO_POP3:	default_radio = private->pop3_radio; break;
-	default:		g_return_val_if_reached(NULL);
-	}
-
-      if (! GTK_WIDGET_IS_SENSITIVE(default_radio))
-	default_radio = private->file_radio;
-
-      gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(default_radio), TRUE);
-    }
+    mn_mailbox_properties_dialog_set_active_type(dialog, selected_type);
     
-  mn_mailbox_properties_dialog_update_sensitivity(dialog);
-
   return GTK_WIDGET(dialog);
 }
 
@@ -163,6 +216,7 @@ mn_mailbox_properties_dialog_private_free (Private *private)
   g_return_if_fail(private != NULL);
 
   g_free(private->uri);
+  g_object_unref(private->store);
   g_free(private);
 }
 
@@ -175,56 +229,146 @@ mn_mailbox_properties_dialog_is_complete (MNMailboxPropertiesDialog *dialog)
   g_return_val_if_fail(MN_IS_MAILBOX_PROPERTIES_DIALOG(dialog), FALSE);
   private = MN_MAILBOX_PROPERTIES_DIALOG_PRIVATE(dialog);
 
-  if (gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(private->file_radio)))
+  switch (mn_mailbox_properties_dialog_get_active_type(dialog))
     {
-      const char *location;
+    case TYPE_AUTODETECT:
+      {
+	const char *location;
+	
+	location = gtk_entry_get_text(GTK_ENTRY(private->location_entry));
+	is_complete = *location != 0;
+      }
+      break;
 
-      location = gtk_entry_get_text(GTK_ENTRY(private->location_entry));
-      is_complete = *location != 0;
-    }
-  else if (GTK_WIDGET_IS_SENSITIVE(private->pop3_radio))
-    {
-      const char *hostname;
-      const char *username;
-      const char *password;
+    case TYPE_POP3:
+#ifdef WITH_POP3
+      {
+	const char *hostname;
+	const char *username;
+	const char *password;
       
-      hostname = gtk_entry_get_text(GTK_ENTRY(private->hostname_entry));
-      username = gtk_entry_get_text(GTK_ENTRY(private->username_entry));
-      password = gtk_entry_get_text(GTK_ENTRY(private->password_entry));
+	hostname = gtk_entry_get_text(GTK_ENTRY(private->pop3_hostname_entry));
+	username = gtk_entry_get_text(GTK_ENTRY(private->pop3_username_entry));
+	password = gtk_entry_get_text(GTK_ENTRY(private->pop3_password_entry));
 
-      is_complete = *hostname != 0 && *username != 0 && *password != 0;
+	is_complete = *hostname != 0 && *username != 0 && *password != 0;
+      }
+#endif /* WITH_POP3 */
+      break;
+
+    case TYPE_GMAIL:
+#ifdef WITH_GMAIL
+      {
+	const char *username;
+	const char *password;
+	
+	username = gtk_entry_get_text(GTK_ENTRY(private->gmail_username_entry));
+	password = gtk_entry_get_text(GTK_ENTRY(private->gmail_password_entry));
+      
+	is_complete = *username != 0 && *password != 0;
+      }
+#endif /* WITH_GMAIL */
+      break;
+
+    default:
+      g_return_val_if_reached(FALSE);
     }
-
+  
   return is_complete;
 }
 
+static Type
+mn_mailbox_properties_dialog_get_active_type (MNMailboxPropertiesDialog *dialog)
+{
+  Private *private;
+  GtkTreeIter iter;
+  Type type = TYPE_AUTODETECT;
+
+  g_return_val_if_fail(MN_IS_MAILBOX_PROPERTIES_DIALOG(dialog), 0);
+  private = MN_MAILBOX_PROPERTIES_DIALOG_PRIVATE(dialog);
+
+  if (gtk_combo_box_get_active_iter(GTK_COMBO_BOX(private->type_combo), &iter))
+    gtk_tree_model_get(GTK_TREE_MODEL(private->store), &iter, COLUMN_TYPE, &type, -1);
+
+  return type;
+}
+
+static void
+mn_mailbox_properties_dialog_set_active_type (MNMailboxPropertiesDialog *dialog,
+					      Type type)
+{
+  Private *private;
+  gboolean valid;
+  GtkTreeIter iter;
+
+  g_return_if_fail(MN_IS_MAILBOX_PROPERTIES_DIALOG(dialog));
+  private = MN_MAILBOX_PROPERTIES_DIALOG_PRIVATE(dialog);
+
+  valid = gtk_tree_model_get_iter_first(GTK_TREE_MODEL(private->store), &iter);
+  while (valid)
+    {
+      Type this_type;
+
+      gtk_tree_model_get(GTK_TREE_MODEL(private->store), &iter, COLUMN_TYPE, &this_type, -1);
+      if (this_type == type)
+	{
+	  gtk_combo_box_set_active_iter(GTK_COMBO_BOX(private->type_combo), &iter);
+	  break;
+	}
+
+      valid = gtk_tree_model_iter_next(GTK_TREE_MODEL(private->store), &iter);
+    }
+}
+
 static void
 mn_mailbox_properties_dialog_update_sensitivity (MNMailboxPropertiesDialog *dialog)
 {
   Private *private;
-  gboolean file_selected;
-  gboolean pop3_selected;
+  Type type;
   gboolean is_complete;
 
   g_return_if_fail(MN_IS_MAILBOX_PROPERTIES_DIALOG(dialog));
   private = MN_MAILBOX_PROPERTIES_DIALOG_PRIVATE(dialog);
 
-  file_selected = gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(private->file_radio));
-  pop3_selected = gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(private->pop3_radio)) && GTK_WIDGET_IS_SENSITIVE(private->pop3_radio);
+  type = mn_mailbox_properties_dialog_get_active_type(dialog);
+  
+  switch (mn_mailbox_properties_dialog_get_active_type(dialog))
+    {
+    case TYPE_AUTODETECT:
+      gtk_widget_show(private->autodetect_vbox);
+      gtk_widget_hide(private->unsupported_label);
+      gtk_widget_hide(private->pop3_vbox);
+      gtk_widget_hide(private->gmail_vbox);
+      break;
 
-  gtk_widget_set_sensitive(private->location_label, file_selected);
-  gtk_widget_set_sensitive(private->location_entry, file_selected);
-  gtk_widget_set_sensitive(private->browse, file_selected);
-
-  gtk_widget_set_sensitive(private->hostname_label, pop3_selected);
-  gtk_widget_set_sensitive(private->hostname_entry, pop3_selected);
-  gtk_widget_set_sensitive(private->port_label, pop3_selected);
-  gtk_widget_set_sensitive(private->port_spin, pop3_selected);
-  gtk_widget_set_sensitive(private->username_label, pop3_selected);
-  gtk_widget_set_sensitive(private->username_entry, pop3_selected);
-  gtk_widget_set_sensitive(private->password_label, pop3_selected);
-  gtk_widget_set_sensitive(private->password_entry, pop3_selected);
+    case TYPE_POP3:
+#ifdef WITH_POP3
+      gtk_widget_show(private->pop3_vbox);
+      gtk_widget_hide(private->unsupported_label);
+#else
+      gtk_widget_show(private->unsupported_label);
+      gtk_widget_hide(private->pop3_vbox);
+#endif /* WITH_POP3 */
+      gtk_widget_hide(private->autodetect_vbox);
+      gtk_widget_hide(private->gmail_vbox);
+      break;
 
+    case TYPE_GMAIL:
+#ifdef WITH_GMAIL
+      gtk_widget_show(private->gmail_vbox);
+      gtk_widget_hide(private->unsupported_label);
+#else
+      gtk_widget_show(private->unsupported_label);
+      gtk_widget_hide(private->gmail_vbox);
+#endif /* WITH_GMAIL */
+      gtk_widget_hide(private->autodetect_vbox);
+      gtk_widget_hide(private->pop3_vbox);
+      break;
+      
+    default:
+      g_return_if_reached();
+    }
+  
   is_complete = mn_mailbox_properties_dialog_is_complete(dialog);
   if (private->apply_button)
     gtk_widget_set_sensitive(private->apply_button, is_complete);
@@ -258,6 +402,7 @@ static void
 mn_mailbox_properties_dialog_fill (MNMailboxPropertiesDialog *dialog)
 {
   Private *private;
+  Type type;
   char *hostname;
   int port;
   char *username;
@@ -269,22 +414,39 @@ mn_mailbox_properties_dialog_fill (MNMailboxPropertiesDialog *dialog)
 
   if (mn_uri_parse_pop(private->uri, &username, &password, &hostname, &port))
     {
-      gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(private->pop3_radio), TRUE);
+      type = TYPE_POP3;
 
-      gtk_entry_set_text(GTK_ENTRY(private->hostname_entry), hostname);
-      gtk_spin_button_set_value(GTK_SPIN_BUTTON(private->port_spin), port);
-      gtk_entry_set_text(GTK_ENTRY(private->username_entry), username);
-      gtk_entry_set_text(GTK_ENTRY(private->password_entry), password);
+#ifdef WITH_POP3
+      gtk_entry_set_text(GTK_ENTRY(private->pop3_hostname_entry), hostname);
+      gtk_spin_button_set_value(GTK_SPIN_BUTTON(private->pop3_port_spin), port);
+      gtk_entry_set_text(GTK_ENTRY(private->pop3_username_entry), username);
+      gtk_entry_set_text(GTK_ENTRY(private->pop3_password_entry), password);
+#endif /* WITH_POP3 */
 
       g_free(hostname);
+      g_free(username);
+      g_free(password);
+    }
+  else if (mn_uri_parse_gmail(private->uri, &username, &password))
+    {
+      type = TYPE_GMAIL;
+
+#ifdef WITH_GMAIL
+      gtk_entry_set_text(GTK_ENTRY(private->gmail_username_entry), username);
+      gtk_entry_set_text(GTK_ENTRY(private->gmail_password_entry), password);
+#endif /* WITH_GMAIL */
+
       g_free(username);
       g_free(password);
     }
   else
     {
-      gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(private->file_radio), TRUE);
+      type = TYPE_AUTODETECT;
+
       gtk_entry_set_text(GTK_ENTRY(private->location_entry), private->uri);
     }
+
+  mn_mailbox_properties_dialog_set_active_type(dialog, type);
 }
 
 void
@@ -310,28 +472,53 @@ mn_mailbox_properties_dialog_get_uri (MNMailboxPropertiesDialog *dialog)
   g_return_val_if_fail(MN_IS_MAILBOX_PROPERTIES_DIALOG(dialog), NULL);
   private = MN_MAILBOX_PROPERTIES_DIALOG_PRIVATE(dialog);
 
-  if (gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(private->file_radio)))
+  switch (mn_mailbox_properties_dialog_get_active_type(dialog))
     {
-      const char *location;
+    case TYPE_AUTODETECT:
+      {
+	const char *location;
+	
+	location = gtk_entry_get_text(GTK_ENTRY(private->location_entry));
+	uri = g_strdup(location);
+      }
+      break;
 
-      location = gtk_entry_get_text(GTK_ENTRY(private->location_entry));
-      uri = g_strdup(location);
+#ifdef WITH_POP3
+    case TYPE_POP3:
+      {
+	const char *hostname;
+	int port;
+	const char *username;
+	const char *password;
+	
+	hostname = gtk_entry_get_text(GTK_ENTRY(private->pop3_hostname_entry));
+	port = gtk_spin_button_get_value_as_int(GTK_SPIN_BUTTON(private->pop3_port_spin));
+	username = gtk_entry_get_text(GTK_ENTRY(private->pop3_username_entry));
+	password = gtk_entry_get_text(GTK_ENTRY(private->pop3_password_entry));
+	
+	uri = mn_uri_build_pop(username, password, hostname, port);
+      }
+      break;
+#endif /* WITH_POP3 */
+
+#ifdef WITH_GMAIL
+    case TYPE_GMAIL:
+      {
+	const char *username;
+	const char *password;
+
+	username = gtk_entry_get_text(GTK_ENTRY(private->gmail_username_entry));
+	password = gtk_entry_get_text(GTK_ENTRY(private->gmail_password_entry));
+	
+	uri = mn_uri_build_gmail(username, password);
+      }
+      break;
+#endif /* TYPE_GMAIL */
+      
+    default:
+      g_return_val_if_reached(NULL);
     }
-  else
-    {
-      const char *hostname;
-      int port;
-      const char *username;
-      const char *password;
-
-      hostname = gtk_entry_get_text(GTK_ENTRY(private->hostname_entry));
-      port = gtk_spin_button_get_value_as_int(GTK_SPIN_BUTTON(private->port_spin));
-      username = gtk_entry_get_text(GTK_ENTRY(private->username_entry));
-      password = gtk_entry_get_text(GTK_ENTRY(private->password_entry));
-
-      uri = mn_uri_build_pop(username, password, hostname, port);
-    }
-
+  
   return uri;
 }
 
@@ -425,34 +612,27 @@ mn_mailbox_properties_dialog_browse_clicked_h (gpointer user_data,
 }
 
 void
-mn_mailbox_properties_dialog_radio_toggled_h (gpointer user_data,
-					      GtkWidget *widget)
+mn_mailbox_properties_dialog_type_changed_h (gpointer user_data,
+					     GtkComboBox *combobox)
 {
   MNMailboxPropertiesDialog *dialog = user_data;
   Private *private;
 
   private = MN_MAILBOX_PROPERTIES_DIALOG_PRIVATE(dialog);
   if (private->mode == MN_MAILBOX_PROPERTIES_DIALOG_MODE_ADD)
-    {
-      if (widget == private->file_radio)
-	selected_radio = RADIO_FILE;
-      else if (widget == private->pop3_radio)
-	selected_radio = RADIO_POP3;
-      else
-	g_return_if_reached();
-    }
-  
+    selected_type = mn_mailbox_properties_dialog_get_active_type(dialog);
+
   mn_mailbox_properties_dialog_update_sensitivity(dialog);
 }
 
 void
-mn_mailbox_properties_dialog_pop3_entry_activate_h (gpointer user_data,
-						    GtkWidget *widget)
+mn_mailbox_properties_dialog_entry_activate_h (gpointer user_data,
+					       GtkEntry *entry)
 {
   GtkWidget *next = user_data;
   GtkWidget *toplevel;
 
-  toplevel = gtk_widget_get_toplevel(widget);
+  toplevel = gtk_widget_get_toplevel(GTK_WIDGET(entry));
   if (GTK_WIDGET_TOPLEVEL(toplevel)
       && GTK_WINDOW(toplevel)->default_widget
       && GTK_WIDGET_IS_SENSITIVE(GTK_WINDOW(toplevel)->default_widget))
@@ -463,7 +643,7 @@ mn_mailbox_properties_dialog_pop3_entry_activate_h (gpointer user_data,
 
 void
 mn_mailbox_properties_dialog_entry_changed_h (gpointer user_data,
-					      GtkWidget *widget)
+					      GtkEditable *editable)
 {
   MNMailboxPropertiesDialog *dialog = user_data;
 
