@@ -19,13 +19,31 @@
 #include "config.h"
 #include <stdlib.h>
 #include <string.h>
+#include <sys/types.h>
+#include <sys/socket.h>
+#include <netinet/in.h>
+#include <arpa/inet.h>
 #include <glib/gi18n-lib.h>
 #include <libgnomevfs/gnome-vfs-utils.h>
 #include "mn-uri.h"
 #include "mn-util.h"
 
+/*** functions ***************************************************************/
+
+static gboolean mn_uri_is_ipv6_address (const char *hostname);
+
 /*** implementation **********************************************************/
 
+static gboolean
+mn_uri_is_ipv6_address (const char *hostname)
+{
+  char dummy[16];		/* sizeof(struct in6_addr) */
+
+  g_return_val_if_fail(hostname != NULL, FALSE);
+
+  return inet_pton(AF_INET6, hostname, dummy) == 1;
+}
+
 char *
 mn_uri_build_pop (const char *username,
 		  const char *password,
@@ -36,9 +54,14 @@ mn_uri_build_pop (const char *username,
   g_return_val_if_fail(password != NULL, NULL);
   g_return_val_if_fail(hostname != NULL, NULL);
 
-  return port == 110
-    ? g_strdup_printf("pop://%s:%s@%s", username, password, hostname)
-    : g_strdup_printf("pop://%s:%s@%s:%i", username, password, hostname, port);
+  if (mn_uri_is_ipv6_address(hostname))
+    return port == 110
+      ? g_strdup_printf("pop://%s:%s@[%s]", username, password, hostname)
+      : g_strdup_printf("pop://%s:%s@[%s]:%i", username, password, hostname, port);
+  else
+    return port == 110
+      ? g_strdup_printf("pop://%s:%s@%s", username, password, hostname)
+      : g_strdup_printf("pop://%s:%s@%s:%i", username, password, hostname, port);
 }
 
 /*
@@ -49,6 +72,8 @@ mn_uri_build_pop (const char *username,
  * We comply with RFC 1738, but for obvious reasons we require <user>
  * and <password>. We do not comply with RFC 2384 because it forbids
  * <password>.
+ *
+ * For IPv6 addresses, we comply with RFC 2732.
  */
 gboolean
 mn_uri_parse_pop (const char *uri,
@@ -57,46 +82,112 @@ mn_uri_parse_pop (const char *uri,
 		  char **hostname,
 		  int *port)
 {
-  const char *_username;
-  char *_password;
-  char *_hostname;
-  char *_port = NULL;
+  char *username_start;
+  char *username_end;
+  char *password_start;
+  char *password_end;
+  char *hostname_start;
+  char *hostname_end;
+  char *port_start = NULL;
 
   g_return_val_if_fail(uri != NULL, FALSE);
 
   if (strncmp(uri, "pop://", 6))
     return FALSE;
 
-  _username = uri + 6;
-  if (strlen(uri) < 6)
+  username_start = (char *) uri + 6;
+  username_end = strchr(username_start, ':');
+  if (! username_end)
     return FALSE;
 
-  _password = strchr(_username, ':');
-  if (! _password++)		/* ++ to skip the colon */
+  password_start = username_end + 1;
+  password_end = strchr(password_start, '@');
+  if (! password_end)
     return FALSE;
 
-  _hostname = strchr(_password, '@');
-  if (! _hostname++)		/* ++ to skip the arobas */
-    return FALSE;
+  hostname_start = password_end + 1;
+  if (*hostname_start == '[')
+    {
+      hostname_end = strchr(++hostname_start, ']');
+      if (hostname_end)
+	{
+	  if (hostname_end[1])
+	    {
+	      if (hostname_end[1] != ':')
+		return FALSE;
+	      port_start = hostname_end + 2;
+	    }
+	}
+      else
+	return FALSE;
+    }
+  else
+    {
+      hostname_end = strchr(hostname_start, ':');
+      if (hostname_end)
+	port_start = hostname_end + 1;
+      else
+	{
+	  hostname_end = strchr(hostname_start, 0);
+	  g_return_val_if_fail(hostname_end != NULL, FALSE);
+	}
+    }
 
-  _port = strchr(_hostname, ':');
-  /* port is optional */
-  if (_port && ! mn_str_isnumeric(++_port))	/* ++ to skip the colon */
-    return FALSE;
-  
-  if ((_password - _username - 1 <= 0)
-      || (_hostname - _password - 1 <= 0)
-      || (_port && _port - _hostname - 1 <= 0))
+  if ((username_end - username_start <= 0)
+      || (password_end - password_start <= 0)
+      || (hostname_end - hostname_start <= 0)
+      || (port_start && ! mn_str_isnumeric(port_start)))
     return FALSE;
 
   if (username)
-    *username = g_strndup(_username, _password - _username - 1);
+    *username = g_strndup(username_start, username_end - username_start);
   if (password)
-    *password = g_strndup(_password, _hostname - _password - 1);
+    *password = g_strndup(password_start, password_end - password_start);
   if (hostname)
-    *hostname = _port ? g_strndup(_hostname, _port - _hostname - 1) : g_strdup(_hostname);
+    *hostname = g_strndup(hostname_start, hostname_end - hostname_start);
   if (port)
-    *port = _port ? atoi(_port) : 110;
+    *port = port_start ? atoi(port_start) : 110;
+
+  return TRUE;
+}
+
+char *
+mn_uri_build_gmail (const char *username, const char *password)
+{
+  g_return_val_if_fail(username != NULL, NULL);
+  g_return_val_if_fail(password != NULL, NULL);
+
+  return g_strdup_printf("gmail://%s:%s", username, password);
+}
+
+gboolean
+mn_uri_parse_gmail (const char *uri, char **username, char **password)
+{
+  char *username_start;
+  char *username_end;
+  char *password_start;
+
+  g_return_val_if_fail(uri != NULL, FALSE);
+
+  if (strncmp(uri, "gmail://", 8))
+    return FALSE;
+
+  username_start = (char *) uri + 8;
+  username_end = strchr(username_start, ':');
+  if (! username_end)
+    return FALSE;
+
+  password_start = username_end + 1;
+  if (! password_start[0])
+    return FALSE;
+
+  if (username_end - username_start <= 0)
+    return FALSE;
+
+  if (username)
+    *username = g_strndup(username_start, username_end - username_start);
+  if (password)
+    *password = g_strdup(password_start);
 
   return TRUE;
 }
@@ -119,6 +210,12 @@ mn_uri_canonicalize (const char *uri)
       g_free(password);
       g_free(hostname);
     }
+  else if (mn_uri_parse_gmail(uri, &username, &password))
+    {
+      canonical_uri = mn_uri_build_gmail(username, password);
+      g_free(username);
+      g_free(password);
+    }
   else
     canonical_uri = gnome_vfs_make_uri_canonical(uri);
 
@@ -182,10 +279,20 @@ mn_uri_format_for_display (const char *uri)
 	  g_free(username);
 	  g_free(hostname);
 	}
-      else
-	name = g_strdup(uri); /* fallback to the URI */
     }
+  if (! name)
+    {
+      char *username;
 
+      if (mn_uri_parse_gmail(uri, &username, NULL))
+	{
+	  name = g_strdup_printf("%s@gmail.com", username);
+	  g_free(username);
+	}
+    }
+  if (! name)
+    name = g_strdup(uri);	/* fallback to the URI */
+  
   return name;
 }
 
